/*
 * Copyright 2016 - 2022 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <ox/std/defines.hpp>

#ifndef OX_BARE_METAL
#include <ox/claw/read.hpp>

#include "typeconv.hpp"

namespace nostalgia::core {

struct Converter {
	virtual bool matches(const ox::String &srcTypeName, int srcTypeVersion, const ox::String &dstTypeName, int dstTypeVersion) noexcept = 0;

	virtual ~Converter() noexcept = default;

	virtual ox::Error convert(const ox::Buffer &pV1Buff, void *pV2) noexcept = 0;
};

struct TileSheetV1ToV2Converter: public Converter {
	constexpr TileSheetV1ToV2Converter() noexcept = default;

	virtual ~TileSheetV1ToV2Converter() noexcept = default;

	bool matches(const ox::String &srcTypeName, int srcTypeVersion, const ox::String &dstTypeName, int dstTypeVersion) noexcept final {
		return srcTypeName == TileSheetV1::TypeName
		    && srcTypeVersion == TileSheetV1::TypeVersion
		    && dstTypeName == TileSheet::TypeName
		    && dstTypeVersion == TileSheet::TypeVersion;
	}

	ox::Error convert(const ox::Buffer &pV1Buff, void *pV2) noexcept final {
		oxRequire(v1, ox::readClaw<TileSheetV1>(pV1Buff));
		auto v2 = static_cast<TileSheet*>(pV2);
		v2->bpp              = v1.bpp;
		v2->subsheet.name    = "Root";
		v2->subsheet.rows    = v1.rows;
		v2->subsheet.columns = v1.columns;
		v2->subsheet.size    = v1.pixels.size();
		v2->defaultPalette   = v1.defaultPalette;
		v2->pal              = v1.pal;
		v2->pixels           = std::move(v1.pixels);
		return OxError(0);
	}
};

static const auto converters = [] {
	ox::Vector<ox::UniquePtr<Converter>, 1> converters;
	converters.emplace_back(new TileSheetV1ToV2Converter());
	return converters;
}();

static auto findConverter(const ox::String &srcTypeName, int srcTypeVersion, const ox::String &dstTypeName, int dstTypeVersion) noexcept -> ox::Result<Converter*> {
	for (auto &c : converters) {
		if (c->matches(srcTypeName, srcTypeVersion, dstTypeName, dstTypeVersion)) {
			return c.get();
		}
	}
	return OxError(1, "Could not find converter");
};

ox::Error convert(const ox::Buffer &srcBuffer, const ox::String &dstTypeName, int dstTypeVersion, void *dstObj) noexcept {
	oxRequire(hdr, ox::readClawHeader(srcBuffer));
	oxRequire(c, findConverter(hdr.typeName, hdr.typeVersion, dstTypeName, dstTypeVersion));
	return c->convert(srcBuffer, dstObj);
}

}
#endif
